<?php

namespace Drupal\http_headers_cleaner\Middleware;

use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\http_headers_cleaner\Service\HttpHeadersCleanerSettings;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\HttpKernelInterface;

/**
 * Middleware removing http headers according to module configuration.
 *
 * @package Drupal\http_headers_cleaner\Middleware
 */
class HttpHeadersCleanerMiddleware implements HttpKernelInterface {

  use StringTranslationTrait;

  /**
   * The kernel.
   *
   * @var \Symfony\Component\HttpKernel\HttpKernelInterface
   */
  protected $httpKernel;

  /**
   * The settings.
   *
   * @var \Drupal\http_headers_cleaner\Service\HttpHeadersCleanerSettings
   */
  protected $settings;

  /**
   * Constructs the HttpHeadersCleanerMiddleware object.
   *
   * @param \Symfony\Component\HttpKernel\HttpKernelInterface $http_kernel
   *   The decorated kernel.
   * @param \Drupal\http_headers_cleaner\Service\HttpHeadersCleanerSettings $settings
   *   The settings.
   */
  public function __construct(
    HttpKernelInterface $http_kernel,
    HttpHeadersCleanerSettings $settings
  ) {
    $this->httpKernel = $http_kernel;
    $this->settings = $settings;
  }

  /**
   * {@inheritdoc}
   */
  public function handle(Request $request, $type = self::MAIN_REQUEST, $catch = TRUE): Response {

    $response = $this->httpKernel->handle($request, $type, $catch);
    if ($type === self::MAIN_REQUEST && $this->settings->isEnabled(HttpHeadersCleanerSettings::TYPE_HEADERS)) {
      $this->cleanHeaders($response);
    }

    return $response;
  }

  /**
   * Clean headers.
   *
   * @param \Symfony\Component\HttpFoundation\Response $response
   *   The response.
   */
  protected function cleanHeaders(Response $response) {
    $headers = $response->headers;
    $headers_array = $headers->all();

    foreach ($this->settings->getConfig(HttpHeadersCleanerSettings::TYPE_HEADERS) as $key => $patternsList) {
      if (is_array($patternsList)) {
        if (isset($headers_array[$key]) && is_array($headers_array[$key])) {
          $clean = array_filter($headers_array[$key], function ($item) use ($patternsList) {
            return !$this->matchPatterns($item, $patternsList);
          });

          $headers->set($key, $clean);
        }
      }
      else {
        $headers->remove($key);
      }
    }

    return $response;
  }

  /**
   * Return true if matches any pattern.
   *
   * @param string $item
   *   The item.
   * @param array $patterns_list
   *   The patterns list.
   */
  protected function matchPatterns($item, array $patterns_list) {

    foreach ($patterns_list as $pattern) {
      $match = [];
      if (preg_match($pattern, $item, $match)) {
        return TRUE;
      }
    }

    return FALSE;
  }

}
